// filefind.cpp

#include "stdafx.h"

IMPLEMENT_DYNAMIC(CFileFind, CObject)

////////////////////////////////////////////////////////////////////////////
// CFileFind implementation

CFileFind::CFileFind()
{
   m_pFoundInfo = NULL;
   m_pNextInfo = NULL;
   m_hContext = NULL;
   m_chDirSeparator = '\\';
}

CFileFind::~CFileFind()
{            
   Close();
}

void CFileFind::Close()
{
   if (m_pFoundInfo != NULL)
   {
      delete m_pFoundInfo;
      m_pFoundInfo = NULL;
   }

   if (m_pNextInfo != NULL)
   {
      delete m_pNextInfo;
      m_pNextInfo = NULL;
   }

   if (m_hContext != NULL)
   {
      CloseContext();
      m_hContext = NULL;
   }
}

void CFileFind::CloseContext()
{
   delete m_hContext;
   return;
}

BOOL CFileFind::FindFile(LPCTSTR pstrName /* = NULL */, DWORD /*dwUnused = 0 */)
{
   Close();
   m_pNextInfo = new struct _find_t;
   m_hContext = new struct _find_t;
   m_bGotLast = FALSE;

   if (pstrName == NULL)
      pstrName = _T("*.*");
   //lstrcpy(((struct _find_t*) m_pNextInfo)->name, pstrName);

   if (0 == _dos_findfirst(pstrName, _A_ARCH | _A_NORMAL | _A_SUBDIR | _A_SYSTEM | _A_HIDDEN, (struct _find_t*) m_hContext))
   {   
      *(struct _find_t*)m_pNextInfo = *(struct _find_t*) m_hContext;
   }
   else
   {
      Close();
      return FALSE;
   }

   LPTSTR pstrRoot = m_strRoot.GetBufferSetLength(_MAX_PATH);
   LPCTSTR pstr = _tfullpath(pstrRoot, pstrName, _MAX_PATH);

   // passed name isn't a valid path but was found by the API
   ASSERT(pstr != NULL);
   if (pstr == NULL)
   {
      m_strRoot.ReleaseBuffer(-1);
      Close();
      return FALSE;
   }
   else
   {
      // find the last forward or backward whack
      LPTSTR pstrBack  = _tcsrchr(pstrRoot, '\\');
      LPTSTR pstrFront = _tcsrchr(pstrRoot, '/');

      if (pstrFront != NULL || pstrBack != NULL)
      {
         if (pstrFront == NULL)
            pstrFront = pstrRoot;
         if (pstrBack == NULL)
            pstrBack = pstrRoot;

         // from the start to the last whack is the root

         if (pstrFront >= pstrBack)
            *pstrFront = '\0';
         else
            *pstrBack = '\0';
      }
      m_strRoot.ReleaseBuffer(-1);
   }

   return TRUE;
}

BOOL CFileFind::MatchesMask(DWORD dwMask) const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   if (m_pFoundInfo != NULL)
      return (!!(((struct _find_t*) m_pFoundInfo)->attrib & dwMask));
   else
      return FALSE;
}

/*
BOOL CFileFind::GetLastAccessTime(FILETIME* pTimeStamp) const
{
   ASSERT(m_hContext != NULL);
   ASSERT(pTimeStamp != NULL);
   ASSERT_VALID(this);

   if (m_pFoundInfo != NULL && pTimeStamp != NULL)
   {
      *pTimeStamp = ((struct _find_t*) m_pFoundInfo)->ftLastAccessTime;
      return TRUE;
   }
   else
      return FALSE;
}

BOOL CFileFind::GetLastWriteTime(FILETIME* pTimeStamp) const
{
   ASSERT(m_hContext != NULL);
   ASSERT(pTimeStamp != NULL);
   ASSERT_VALID(this);

   if (m_pFoundInfo != NULL && pTimeStamp != NULL)
   {
      *pTimeStamp = ((struct _find_t*) m_pFoundInfo)->ftLastWriteTime;
      return TRUE;
   }
   else
      return FALSE;
}

BOOL CFileFind::GetCreationTime(FILETIME* pTimeStamp) const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   if (m_pFoundInfo != NULL && pTimeStamp != NULL)
   {
      *pTimeStamp = ((struct _find_t*) m_pFoundInfo)->ftCreationTime;
      return TRUE;
   }
   else
      return FALSE;
}
*/
BOOL CFileFind::GetLastAccessTime(CTime& refTime) const
{
   return GetLastWriteTime(refTime);
}

BOOL CFileFind::GetCreationTime(CTime& refTime) const
{
   return GetLastWriteTime(refTime);
}

BOOL CFileFind::GetLastWriteTime(CTime& refTime) const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   if (m_pFoundInfo != NULL)
   {
      refTime = CTime(
      ((struct _find_t*) m_pFoundInfo)->wr_date,
      ((struct _find_t*) m_pFoundInfo)->wr_time
      );
      return TRUE;
   }
   else
      return FALSE;
}

BOOL CFileFind::IsDots() const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   // return TRUE if the file name is "." or ".." and
   // the file is a directory

   BOOL bResult = FALSE;
   if (m_pFoundInfo != NULL && IsDirectory())
   {
      struct _find_t* pFindData = (struct _find_t*) m_pFoundInfo;
      if (pFindData->name[0] == '.')
      {
         if (pFindData->name[1] == '\0' ||
            (pFindData->name[1] == '.' &&
             pFindData->name[2] == '\0'))
         {
            bResult = TRUE;
         }
      }
   }

   return bResult;
}

BOOL CFileFind::FindNextFile()
{
   ASSERT(m_hContext != NULL);

   if (m_hContext == NULL)
      return FALSE;
   if (m_pFoundInfo == NULL)
      m_pFoundInfo = new struct _find_t;

   ASSERT_VALID(this);

   void* pTemp = m_pFoundInfo;
   m_pFoundInfo = m_pNextInfo;
   m_pNextInfo = pTemp;

   BOOL bOK = (0 == _dos_findnext((struct _find_t*)m_hContext));
   if (bOK)
   {
      *(struct _find_t*)m_pNextInfo = *(struct _find_t*)m_hContext;
   }
   return bOK;
}

CString CFileFind::GetFileURL() const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   CString strResult("file://");
   strResult += GetFilePath();
   return strResult;
}

CString CFileFind::GetRoot() const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   return m_strRoot;
}

CString CFileFind::GetFilePath() const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   CString strResult = m_strRoot;
   if (strResult[strResult.GetLength()-1] != '\\' &&
      strResult[strResult.GetLength()-1] != '/')
      strResult += m_chDirSeparator;
   strResult += GetFileName();
   return strResult;
}

CString CFileFind::GetFileTitle() const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   CString strFullName = GetFileName();
   CString strResult;

   _tsplitpath(strFullName, NULL, NULL, strResult.GetBuffer(_MAX_PATH), NULL);
   strResult.ReleaseBuffer();
   return strResult;
}

CString CFileFind::GetFileName() const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   CString ret;

   if (m_pFoundInfo != NULL)
      ret = ((struct _find_t*) m_pFoundInfo)->name;
   return ret;
}

DWORD CFileFind::GetLength() const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   if (m_pFoundInfo != NULL)
      return ((struct _find_t*) m_pFoundInfo)->size;
   else
      return 0;
}

#if defined(_X86_) || defined(_ALPHA_)
__int64 CFileFind::GetLength64() const
{
   ASSERT(m_hContext != NULL);
   ASSERT_VALID(this);

   if (m_pFoundInfo != NULL)
      return ((struct _find_t*) m_pFoundInfo)->nFileSizeLow +
            (((struct _find_t*) m_pFoundInfo)->nFileSizeHigh << 32);
   else
      return 0;
}
#endif

#ifdef _DEBUG
void CFileFind::Dump(CDumpContext& dc) const
{
   CObject::Dump(dc);
   dc << "\nm_hContext = " << (unsigned long) m_hContext;
}

void CFileFind::AssertValid() const
{
   // if you trip the ASSERT in the else side, you've called
   // a Get() function without having done at least one
   // FindNext() call

   if (m_hContext == NULL)
      ASSERT(m_pFoundInfo == NULL && m_pNextInfo == NULL);
   else
      ASSERT(m_pFoundInfo != NULL && m_pNextInfo != NULL);

}
#endif

#ifdef AFX_INIT_SEG
#pragma code_seg(AFX_INIT_SEG)
#endif
