// mfczip.inl

#ifdef __AFX_H__

/////////////////////////////////////////////////////////////////////////////
// CMFCZipFile

class CMFCZipFile : public CFile
{
public:
   static voidpf ZCALLBACK open_file_func(voidpf opaque, const char* filename, int mode);
   static voidpf ZCALLBACK open_memfile_func(voidpf opaque, const char* filename, int mode);
   static uLong  ZCALLBACK read_file_func(voidpf opaque, voidpf stream, void* buf, uLong size);
   static uLong  ZCALLBACK write_file_func(voidpf opaque, voidpf stream, const void* buf, uLong size);
   static ZPOS_T ZCALLBACK tell_file_func(voidpf opaque, voidpf stream);
   static long   ZCALLBACK seek_file_func(voidpf opaque, voidpf stream, ZOFF_T offset, int origin);
   static int    ZCALLBACK close_file_func(voidpf opaque, voidpf stream);
   static int    ZCALLBACK error_file_func(voidpf opaque, voidpf stream);
};

/*static*/ inline voidpf ZCALLBACK CMFCZipFile::open_file_func(voidpf opaque, const char* filename, int mode)
{
   CFile* pFile = (CFile*)opaque;
   if (!pFile->IsKindOf(RUNTIME_CLASS(CMemFile)))
   {
      USES_CONVERSION;
      DWORD dwFlags = 0;

      if ((mode & ZLIB_FILEFUNC_MODE_READWRITEFILTER)==ZLIB_FILEFUNC_MODE_READ)
      {
         dwFlags|=CFile::modeRead | CFile::typeBinary;
      }
      else if (mode & ZLIB_FILEFUNC_MODE_EXISTING)
      {
         dwFlags|=CFile::modeRead | CFile::typeBinary;
      }
      else if (mode & ZLIB_FILEFUNC_MODE_CREATE)
      {
         dwFlags|=CFile::modeCreate | CFile::modeReadWrite | CFile::typeBinary;
      }
      if (!pFile->Open(A2CT(filename), dwFlags))
      {
         pFile = NULL;
      }
   }
   return pFile;
}

/*static*/ inline uLong  ZCALLBACK CMFCZipFile::read_file_func(voidpf opaque, voidpf, void* buf, uLong size)
{
   return ((CFile*)opaque)->Read(buf, size);
}

/*static*/ inline uLong  ZCALLBACK CMFCZipFile::write_file_func(voidpf opaque, voidpf, const void* buf, uLong size)
{
   TRY
   {
      ((CFile*)opaque)->Write(buf, size);
      return size;
   }
   CATCH_ALL(e)
   {
   }
   END_CATCH_ALL
   return 0;
}

/*static*/ inline ZPOS_T ZCALLBACK CMFCZipFile::tell_file_func(voidpf opaque, voidpf)
{
   return ((CFile*)opaque)->GetPosition();
}

/*static*/ inline long   ZCALLBACK CMFCZipFile::seek_file_func(voidpf opaque, voidpf, ZOFF_T offset, int origin)
{
   enum CFile::SeekPosition fseek_origin;
   switch (origin)
   {
      case ZLIB_FILEFUNC_SEEK_CUR :
         fseek_origin = CFile::current;
         break;
      case ZLIB_FILEFUNC_SEEK_END :
         fseek_origin = CFile::end;
         break;
      case ZLIB_FILEFUNC_SEEK_SET :
         fseek_origin = CFile::begin;
         break;
      default: 
         return -1;
   }
   ((CFile*)opaque)->Seek((DWORD)offset, fseek_origin);
   return 0;
}

/*static*/ inline int ZCALLBACK CMFCZipFile::close_file_func(voidpf opaque, voidpf)
{
   ((CFile*)opaque)->Close();
   return 0;
}

/*static*/ inline int    ZCALLBACK CMFCZipFile::error_file_func(voidpf opaque, voidpf)
{
   ((CFile*)opaque)->Abort();
   return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CMFCZipBase

inline CMFCZipBase::CMFCZipBase(size_t buf_size) : m_bContinue(TRUE), m_error_count(0)
{
   m_buffer.SetSize(buf_size);
   m_filefunc.zopen_file  = CMFCZipFile::open_file_func ;
   m_filefunc.zread_file  = CMFCZipFile::read_file_func ;
   m_filefunc.zwrite_file = CMFCZipFile::write_file_func;
   m_filefunc.ztell_file  = CMFCZipFile::tell_file_func ;
   m_filefunc.zseek_file  = CMFCZipFile::seek_file_func ;
   m_filefunc.zclose_file = CMFCZipFile::close_file_func;
   m_filefunc.zerror_file = CMFCZipFile::error_file_func;
   m_filefunc.opaque = NULL;
}

inline void CMFCZipBase::SetFileFunc(CFile* pFile)
{
   m_filefunc.opaque = pFile;
}

inline void CMFCZipBase::OnError(LPCTSTR /*lpszFilePath*/)
{
   m_bContinue = TRUE;
}

inline int CMFCZipBase::WriteStatus (LPCTSTR fmt, ...)
{
   int res;
   va_list args;

   va_start(args, fmt);
   res = WriteStatusV(fmt, args);
   va_end(args);
   return res;
}

inline int CMFCZipBase::WriteStatusV(LPCTSTR fmt, va_list argptr)
{
   return _vtprintf(fmt, argptr);
}

/////////////////////////////////////////////////////////////////////////////
// CMFCZip

inline CMFCZip::CMFCZip(size_t buf_size) : CZip(), CMFCZipBase(buf_size)
{ 
}

inline CMFCZip::~CMFCZip()
{
}

inline BOOL CMFCZip::Open(LPCTSTR lpszFilePath, BOOL bAppend, const char** globalcomment)
{
   USES_CONVERSION;
   m_bContinue = TRUE, m_error_count = 0;
   return CZip::Open(T2CA(lpszFilePath), bAppend, globalcomment, (m_filefunc.opaque != NULL) ? &m_filefunc : NULL);
}

inline BOOL CMFCZip::Open(CFile* file, BOOL bAppend, const char** globalcomment)
{
   SetFileFunc(file);
   return CMFCZip::Open(_T("MEMFILE"), bAppend, globalcomment);
}

inline BOOL CMFCZip::AddNew(FILE* file,
            LPCTSTR filenameinzip,
            LPCTSTR comment,
            int level,
            int method,             
            const void* extrafield_local,
            uInt size_extrafield_local,
            const void* extrafield_global,
            uInt size_extrafield_global)
{
   USES_CONVERSION;

   CString str = filenameinzip;
#if _MFC_VER > 0x0253
   str.Replace('\\', '/');
#endif
   return CZip::AddNew(file,
            T2CA(str),
            m_buffer.GetData(),
            m_buffer.GetSize(),
            T2CA(comment),
            level,
            method,              
            extrafield_local,
            size_extrafield_local,
            extrafield_global,
            size_extrafield_global);
}

inline BOOL CMFCZip::OpenNew(LPCTSTR filename,
            const zip_fileinfo* zipfi,
            LPCTSTR comment,
            int level,
            int method,
            const void* extrafield_local,
            uInt size_extrafield_local,
            const void* extrafield_global,
            uInt size_extrafield_global)
{
   USES_CONVERSION;
   return CZip::OpenNew(T2CA(filename), zipfi, 
            T2CA(comment), level,
            method,
            extrafield_local,
            size_extrafield_local,
            extrafield_global,
            size_extrafield_global);
}

inline BOOL CMFCZip::AddNewAccept(LPCTSTR /*lpszFilePath*/) const
{
   return TRUE;
}

inline BOOL CMFCZip::AddNew(LPCTSTR lpszFilter, BOOL bRecursive, LPCTSTR lpszPrefix)
{
   CFileFind ff;
   BOOL bContinue = ff.FindFile(lpszFilter, 0);
   BOOL bOK = bContinue && (m_buffer.GetSize() != 0);
   if (bOK)
   {
      for (int i = 0; bContinue && m_bContinue; )
      {
         bContinue = ff.FindNextFile();
         if (ff.IsDots())
         {
            continue;
         }
         const CString strFullPath = ff.GetFilePath();
         if (!AddNewAccept(strFullPath))
         {
            continue;
         }
         if (ff.IsDirectory())
         {
            if (bRecursive)
            {
               CString strFilter, strPrefix;
               _tmakepath(strFilter.GetBuffer(_MAX_PATH), NULL, strFullPath, _T("*.*"), NULL);
               strFilter.ReleaseBuffer();
               _tmakepath(strPrefix.GetBuffer(_MAX_PATH), NULL, lpszPrefix, ff.GetFileName(), NULL);
               strPrefix.ReleaseBuffer();
               AddNew(strFilter, bRecursive, strPrefix);
            }
         }
         else
         {
            CStdioFile infile;
            BOOL bSuccess = infile.Open(strFullPath, CFile::modeRead | CFile::typeBinary);
            CString strTitle;
            _tmakepath(strTitle.GetBuffer(_MAX_PATH), NULL, lpszPrefix, ff.GetFileName(), NULL);
            strTitle.ReleaseBuffer();
            WriteStatus(_T("Compressing %s ..."), strTitle.operator LPCTSTR());
            if (bSuccess)
            {
               bSuccess = AddNew(infile.m_pStream, strTitle);
            }
            WriteStatus(_T(" %s\n\r"), bSuccess ? _T("ok") : _T("failed"));
            if (!bSuccess)
            {
               m_error_count++;
               OnError(strFullPath);
            }
         }
         i++;
      }
   }
   else
   {
      m_error_count++;
   }
   return (0 == m_error_count);
}

/////////////////////////////////////////////////////////////////////////////
// CMFCUnzip

inline CMFCUnzip::CMFCUnzip(size_t buf_size) : CUnzip(), CMFCZipBase(buf_size)
{ 
}

inline CMFCUnzip::~CMFCUnzip()
{
}

inline BOOL CMFCUnzip::Open(LPCTSTR lpszFilePath)
{
   USES_CONVERSION;
   m_bContinue = TRUE, m_error_count = 0;
   return CUnzip::Open(T2CA(lpszFilePath), (m_filefunc.opaque != NULL) ? &m_filefunc : NULL);
}

inline BOOL CMFCUnzip::Open(CFile* file)
{
   SetFileFunc(file);
   return Open(_T("MEMFILE"));
}

inline BOOL CMFCUnzip::GetGlobalComment(CString* str)
{
   BOOL bOK;
#ifdef _UNICODE
   char szComment[_MAX_PATH];
   bOK = CUnzip::GetGlobalComment(szComment, _countof(szComment));
   if (bOK)
   {
      str->operator=(szComment);
   }
#else
   bOK = CUnzip::GetGlobalComment(str->GetBuffer(_MAX_PATH), _MAX_PATH);
   str->ReleaseBuffer();
#endif
   return bOK;
}

inline BOOL CMFCUnzip::GetCurrentFileInfo(unz_file_info *pfile_info,
                 CString* strFileName,
                 void *extraField,
                 uLong extraFieldBufferSize,
                 CString* strComment) const
{
#ifdef _UNICODE
   char szComment[_MAX_PATH], szFileName[_MAX_PATH];
   BOOL bOK = CUnzip::GetCurrentFileInfo(pfile_info,
                 (strFileName != NULL) ? szFileName : NULL, _countof(szFileName),
                 extraField,
                 extraFieldBufferSize,
                 (strComment != NULL) ? szComment : NULL, _countof(szComment));
   if (bOK)
   {
      if (strFileName != NULL) strFileName->operator=(szFileName);
      if (strComment  != NULL) strComment ->operator=(szComment );
   }
#else
   BOOL bOK = CUnzip::GetCurrentFileInfo(pfile_info,
                 (strFileName != NULL) ? strFileName->GetBuffer(_MAX_PATH) : NULL, 
                 (strFileName != NULL) ? _MAX_PATH : 0,
                 extraField,
                 extraFieldBufferSize,
                 (strComment != NULL) ? strComment->GetBuffer(_MAX_PATH) : NULL, 
                 (strComment != NULL) ? _MAX_PATH : 0);
   if (strFileName != NULL) strFileName->ReleaseBuffer();
   if (strComment  != NULL) strComment ->ReleaseBuffer();
#endif
   return bOK;   
}

inline BOOL CMFCUnzip::ExtractAccept(LPCTSTR write_filename, BOOL bOverwrite)
{
   BOOL bOK = TRUE;
   if (!bOverwrite)
   {
      FILE* file = _tfopen(write_filename, _T("rb"));
      BOOL bExisting = (file != NULL);
      if (file != NULL) fclose(file);
      bOK = !bExisting;
   }
   return bOK;
}

inline BOOL CMFCUnzip::Extract(BOOL bFullPath, BOOL bOverwrite, LPCTSTR lpszDst)
{
   BOOL bOK = TRUE;
   for (BOOL bContinue = GoToFirstFile(); m_bContinue && bContinue; bContinue = GoToNextFile())
   {
      CString src_filename;
      unz_file_info info;
      bOK = GetCurrentFileInfo(&info,&src_filename);
      if (!bOK)
      {
         break;
      }
      LPTSTR short_filename;

      for (LPTSTR p = short_filename = (LPTSTR)src_filename.operator LPCTSTR();
           (*p) != '\0'; 
           p++)
      {
         if (((*p)=='/') || ((*p)=='\\'))
         {
            short_filename = p+1;
         }
      }

      if ((*short_filename)=='\0')
      {
         if (bFullPath)
         {
            WriteStatus(_T("creating directory: %s\n"),src_filename.operator LPCTSTR());
            xx_tmkdir(src_filename);
         }
      }
      else
      {
         bOK = OpenCurrentFile();
         if (bOK)
         {
            CString write_filename;
            _tmakepath(write_filename.GetBuffer(_MAX_PATH), NULL, lpszDst, 
               bFullPath ? src_filename.operator LPCTSTR() : short_filename, NULL);
            write_filename.ReleaseBuffer();
#if _MFC_VER > 0x0253
            write_filename.Replace('/', '\\');
#endif
            CString strResult;
            
            WriteStatus(_T("Extracting %s ..."), write_filename.operator LPCTSTR());
            if (ExtractAccept(write_filename, bOverwrite))
            {
               FILE* file = _tfopen(write_filename,_T("wb"));

               /* some zipfile don't contain directory alone before file */
               if (    (file == NULL) 
                    && bFullPath
                    && (short_filename != src_filename.operator LPCTSTR()))
               {
                  CString strDrive, strDir;
                  _tsplitpath(write_filename, strDrive.GetBuffer(_MAX_PATH), strDir.GetBuffer(_MAX_PATH), NULL, NULL); 
                  strDrive.ReleaseBuffer(), strDir.ReleaseBuffer();
                  xx_tmkdir(strDrive + strDir);
                  file = _tfopen(write_filename, _T("wb"));
               }
               BOOL bWrite = (file != NULL);
               if (bWrite)
               {
                  bWrite = ExtractCurrentFile(file);
                  fclose(file);
               }
               if (bWrite) 
               {
                  CUnzip::change_file_date(write_filename,info.dosDate, info.tmu_date);
               }
               else
               {
                  m_error_count++;
                  OnError(write_filename);
               }
               strResult = bWrite ? _T("ok") : _T("failed");
            }
            else
            {
               strResult = _T("skipped");
            }
            WriteStatus(_T(" %s\r\n"), strResult.operator LPCTSTR());
            CloseCurrentFile();
         }
      }
   }
   return bOK;
}

inline BOOL CMFCUnzip::ExtractCurrentFile(FILE* file)
{
   return CUnzip::ExtractCurrentFile(file, m_buffer.GetData(), m_buffer.GetSize());
}

inline BOOL CMFCUnzip::ExtractCurrentFile(CFile* file)
{
   for (BOOL bOK = TRUE; bOK; )
   {
      size_t bytes = ReadCurrentFile(m_buffer.GetData(), m_buffer.GetSize());
      if (bytes < 0) 
      {
         _tprintf(_T("error with zipfile in ReadCurrentFile\n"));
         bOK = FALSE;
      } 
      else if (bytes > 0)
      {
         file->Write(m_buffer.GetData(), bytes);
      }
      else
      {
         break;
      }
   }
   return bOK;
}

/////////////////////////////////////////////////////////////////////////////
// CZipFileFind

inline CZipFileFind::CZipFileFind(CMFCUnzip* pArchive) : CFileFind(), m_index(-1)
{
   ASSERT(pArchive != NULL);
#ifdef _WIN32
   m_hContext = (HANDLE)pArchive;
#else
   m_hContext = pArchive;
#endif
   m_chDirSeparator = '\\';
}

inline CZipFileFind::~CZipFileFind()
{
   Close();
}

inline BOOL CZipFileFind::FindFile(LPCTSTR /*pstrName = NULL */, DWORD /*dwFlags = 0 */)
{
   ASSERT(m_hContext != NULL);

   if (m_hContext == NULL)
      return FALSE;

#ifdef _WIN32
   HANDLE temp = m_hContext;
#else
   void* temp = m_hContext;
#endif
   Close();   
   m_hContext = temp;

   m_pNextInfo = new FILEINFO;
   m_bGotLast = FALSE;

   BOOL bOK = GetArchive()->GoToFirstFile();
   if (bOK)
   {
      m_index = 0;
      VERIFY(FillFindData((FILEINFO*)m_pNextInfo));
   }
   return bOK;
}

inline BOOL CZipFileFind::FillFindData(FILEINFO* dst)
{
   memset(dst, 0, sizeof(*dst));

   CString strFileName;
   BOOL bOK = GetArchive()->GetCurrentFileInfo(dst, &strFileName);
   if (bOK)
   {
#if _MFC_VER > 0x0253
      if (m_chDirSeparator != '/')
      {
         strFileName.Replace('/', m_chDirSeparator);
      }
#endif
#ifdef _WIN32
      _tcsncpy(dst->cFileName, strFileName, _countof(dst->cFileName));
      dst->nFileSizeHigh = 0, dst->nFileSizeLow = dst->uncompressed_size;
      SYSTEMTIME st;
      CTime t (HIWORD(dst->dosDate), LOWORD(dst->dosDate), -1);
      t+=_timezone;
      VERIFY(t.GetAsSystemTime(st));
      if (bOK)
      {
         FILETIME   ft;
         bOK = SystemTimeToFileTime(&st, &ft);
         if (bOK)
         {   
            dst->ftCreationTime = dst->ftLastAccessTime = dst->ftLastWriteTime = ft;
         }
      }
#else
      _tcsncpy(dst->name, strFileName, _countof(dst->name));
      dst->size = dst->uncompressed_size;
      dst->wr_time = HIWORD(dst->dosDate), dst->wr_date = LOWORD(dst->dosDate);
#endif

   }
   return bOK;
}

inline BOOL CZipFileFind::FindNextFile()
{
   ASSERT(m_hContext != NULL);
   if (m_hContext == NULL)
      return FALSE;

   if (m_pFoundInfo == NULL)
   {
      m_pFoundInfo = new FILEINFO;
   }

   ASSERT_VALID(this);
   void* pTemp = m_pFoundInfo;
   m_pFoundInfo = m_pNextInfo;
   m_pNextInfo = pTemp;

   m_index++;
   //ASSERT(m_index < GetArchive()->GetGlobalCount());
   BOOL bOK = GetArchive()->GoToNextFile();
   VERIFY(FillFindData((FILEINFO*)m_pNextInfo));
   return bOK;
}

inline BOOL CZipFileFind::GetCurrentFileInfo(unz_file_info *p) const
{
   *p = *((FILEINFO*)m_pFoundInfo);
   return TRUE;
}

inline CString CZipFileFind::GetFilePath(void) const
{
#ifdef _WIN32
   return ((FILEINFO*)m_pFoundInfo)->cFileName;
#else
   return ((FILEINFO*)m_pFoundInfo)->name;
#endif
}

inline void CZipFileFind::CloseContext()
{
}

#endif // __AFX_H__
