// unzip.inl

/////////////////////////////////////////////////////////////////////////////
// CUnzip

inline CUnzip::CUnzip() : m_handle(NULL)
{
}

inline BOOL CUnzip::Open(const char* lpszFilePath, zlib_filefunc_def* def)
{
   _ASSERTE(m_handle == NULL);
   m_handle = ::unzOpen2(lpszFilePath, def);
   return (m_handle != NULL);
}

inline BOOL CUnzip::IsOpen(void) const
{
   return (m_handle != NULL);
}

inline CUnzip::~CUnzip()
{
   if (IsOpen()) Close();
}

inline void CUnzip::Close()
{
   _ASSERTE(m_handle != NULL);
   ::unzClose(m_handle);
   m_handle = NULL;
}

inline BOOL CUnzip::GetGlobalComment(char *szComment, uLong uSizeBuf) const
{
   _ASSERTE(m_handle != NULL);
   return (UNZ_OK == ::unzGetGlobalComment(m_handle, szComment, uSizeBuf));
}

inline BOOL CUnzip::GetGlobalInfo(unz_global_info * p) const
{
   _ASSERTE(m_handle != NULL);
   return (UNZ_OK == ::unzGetGlobalInfo(m_handle, p));
}

inline int CUnzip::GetGlobalCount(void) const
{
   _ASSERTE(m_handle != NULL);
   unz_global_info info;
   return (::unzGetGlobalInfo(m_handle, &info) == UNZ_OK) ? info.number_entry : 0;
}

inline BOOL CUnzip::GetCurrentFileInfo(unz_file_info *pfile_info,
                    char *szFileName,
                    uLong fileNameBufferSize,
                    void *extraField,
                    uLong extraFieldBufferSize,
                    char *szComment,
                    uLong commentBufferSize) const
{
   _ASSERTE(m_handle != NULL);
   return (UNZ_OK == ::unzGetCurrentFileInfo(m_handle,
                    pfile_info,
                    szFileName,
                    fileNameBufferSize,
                    extraField,
                    extraFieldBufferSize,
                    szComment,
                    commentBufferSize));
}

inline BOOL CUnzip::GoToNextFile(void)
{
   _ASSERTE(m_handle != NULL);
   return (UNZ_OK == ::unzGoToNextFile(m_handle));
}

inline BOOL CUnzip::GoToFirstFile(void)
{
   return (UNZ_OK == ::unzGoToFirstFile(m_handle));
}

inline BOOL CUnzip::CloseCurrentFile()
{
   _ASSERTE(m_handle != NULL);
   return (UNZ_OK == ::unzCloseCurrentFile(m_handle));
}

inline int CUnzip::ReadCurrentFile(voidp buf, size_t len)
{
   _ASSERTE(m_handle != NULL);
   return ::unzReadCurrentFile(m_handle, buf, len);
}

inline BOOL CUnzip::OpenCurrentFile(const char* password)
{
   _ASSERTE(m_handle != NULL);
   return (UNZ_OK == ::unzOpenCurrentFilePassword(m_handle, password));
}

/*static*/inline BOOL CUnzip::change_file_date(LPCTSTR filename,uLong dosdate,tm_unz tmu_date)
{  
   BOOL bOK;
#ifdef _WIN32
   HANDLE hFile = CreateFile(filename,GENERIC_READ | GENERIC_WRITE, 0,NULL,OPEN_EXISTING,0,NULL);
   bOK = (hFile != INVALID_HANDLE_VALUE);
   if (bOK)
   {
      FILETIME ftm,ftLocal,ftLastAcc;
      bOK = GetFileTime(hFile,NULL,&ftLastAcc,NULL);
      DosDateTimeToFileTime((WORD)(dosdate>>16),(WORD)dosdate,&ftLocal);
      LocalFileTimeToFileTime(&ftLocal,&ftm);
      SetFileTime(hFile,&ftm,&ftLastAcc,&ftm);
      CloseHandle(hFile);
   }
#else
   FILE* file = _tfopen(filename, _T("rw"));
   bOK = (file != NULL);
   if (bOK)
   {
      bOK = (0 == _dos_setftime(_fileno(file), HIWORD(dosdate), LOWORD(dosdate)));
      fclose(file);
   }
#endif
   return bOK;
}

inline BOOL CUnzip::ExtractCurrentFile(FILE* file, void* buf, size_t size_buf)
{
   for (BOOL bOK = TRUE; bOK; )
   {
      int bytes = ReadCurrentFile(buf,size_buf);
      if (bytes < 0) 
      {
         _tprintf(_T("error with zipfile in ReadCurrentFile\n"));
         bOK = FALSE;
      } 
      else if (bytes > 0)
      {
         if (fwrite(buf,bytes,1,file)!=1)
         {
            _tprintf(_T("error in writing extracted file\n"));
            bOK = FALSE;
         }
      }
      else
      {
         break;
      }
   }
   return bOK;
}

inline BOOL CUnzip::Attach(unzFile handle)
{
   _ASSERTE(m_handle == NULL);
   m_handle = handle;
   return (handle != NULL);
}

inline unzFile CUnzip::Detach(void)
{
   _ASSERTE(m_handle != NULL);
   unzFile handle = m_handle;
   m_handle = NULL;
   return handle;
}

inline CUnzip::operator unzFile(void)
{
   return m_handle;
}

#ifdef _unzx_H

inline BOOL CUnzip::Validate(const char* password, void* buffer, size_t buf_size)
{
   _ASSERTE(m_handle != NULL);
   return (UNZ_OK == ::unzValidate(m_handle, password, buffer, buf_size));
}

inline BOOL CUnzip::Attach(voidp stream, zlib_filefunc_def* api)
{
   _ASSERTE(m_handle == NULL);
   m_handle = ::unzAttach(stream, api);
   return (m_handle != NULL);
}

#endif // _unzx_H

/////////////////////////////////////////////////////////////////////////////
