// zip.inl

/////////////////////////////////////////////////////////////////////////////
// CZip

inline CZip::CZip() : m_handle(NULL)
{
}

inline BOOL CZip::Open(const char* lpszFilePath, BOOL bAppend, const char** globalcomment, zlib_filefunc_def* pzlib_filefunc_def)
{
   _ASSERTE(m_handle == NULL);
   m_handle = ::zipOpen2(lpszFilePath, bAppend, globalcomment, pzlib_filefunc_def);
   return (m_handle != NULL);
}

inline CZip::~CZip()
{
   if (IsOpen()) Close();
}

inline void CZip::Close()
{
   _ASSERTE(m_handle != NULL);
   ::zipClose(m_handle, NULL);
   m_handle = NULL;
}

inline BOOL CZip::IsOpen(void) const
{
   return (m_handle != NULL);
}

inline BOOL CZip::filetime(LPCTSTR f, tm_zip* tmzip, DWORD* dt)
{  
   BOOL bOK = FALSE;
#ifdef _WIN32
   WIN32_FIND_DATA ff32;
   HANDLE handle = ::FindFirstFile(f,&ff32);
   bOK = (handle != INVALID_HANDLE_VALUE);
   if (bOK)
   {
      FILETIME ftLocal;
      ::FileTimeToLocalFileTime(&(ff32.ftLastWriteTime),&ftLocal);
      ::FileTimeToDosDateTime(&ftLocal,((LPWORD)dt)+1,((LPWORD)dt)+0);
      ::FindClose(handle);
   }
#else
  struct stat s;        /* results of stat() */
  struct tm* filedate;
  time_t tm_t=0;
  
  if (strcmp(f,"-")!=0)
  {
    if (stat(f,&s)==0)
    {
      tm_t = s.st_mtime;
      bOK = TRUE;
    }
  }
  filedate = localtime(&tm_t);

  tmzip->tm_sec  = filedate->tm_sec;
  tmzip->tm_min  = filedate->tm_min;
  tmzip->tm_hour = filedate->tm_hour;
  tmzip->tm_mday = filedate->tm_mday;
  tmzip->tm_mon  = filedate->tm_mon ;
  tmzip->tm_year = filedate->tm_year;
#endif
   return bOK;
}

#ifndef _INC_IO
   #include <io.h>
#endif

inline BOOL CZip::filetime(FILE* file, tm_zip * tmzip, DWORD* dt)
{  
   BOOL bOK = FALSE;
#ifdef _WIN32
   FILETIME ftModify;
   bOK = ::GetFileTime((HANDLE)_get_osfhandle((int)_fileno(file)), NULL, NULL, &ftModify);
   if (bOK)
   {
      FILETIME ftLocal;
      FileTimeToLocalFileTime(&ftModify,&ftLocal);
      FileTimeToDosDateTime(&ftLocal,((LPWORD)dt)+1,((LPWORD)dt)+0);
   }
#else
  struct _stat s;        /* results of stat() */
  struct tm* filedate;
  time_t tm_t=0;
  
 if (_fstat(_fileno(file),&s)==0)
 {
   tm_t = s.st_mtime;
   bOK = TRUE;
 }
  filedate = localtime(&tm_t);

  tmzip->tm_sec  = filedate->tm_sec;
  tmzip->tm_min  = filedate->tm_min;
  tmzip->tm_hour = filedate->tm_hour;
  tmzip->tm_mday = filedate->tm_mday;
  tmzip->tm_mon  = filedate->tm_mon ;
  tmzip->tm_year = filedate->tm_year;
#endif
   return bOK;
}

inline BOOL CZip::OpenNew(const char* filename,
            const zip_fileinfo* zipfi,
            const char* comment,
            int level,
            int method,
            const void* extrafield_local,
            uInt size_extrafield_local,
            const void* extrafield_global,
            uInt size_extrafield_global)
{
   _ASSERTE(m_handle != NULL);
   return (ZIP_OK == ::zipOpenNewFileInZip(m_handle, filename, zipfi, extrafield_local, size_extrafield_local, 
               extrafield_global, size_extrafield_global, comment,
               (level != 0) ? method : 0,
               level));
}

inline BOOL CZip::CloseNew()
{
   _ASSERTE(m_handle != NULL);
   return (ZIP_OK == ::zipCloseFileInZip(m_handle));
}

inline BOOL CZip::WriteNew(const voidp p, size_t len)
{
   _ASSERTE(m_handle != NULL);
   return (::zipWriteInFileInZip(m_handle, p, len) >= 0);
}

inline BOOL CZip::AddNew(FILE* file,
                         const char* filenameinzip,
            void* buf,
            size_t buf_size,
            const char* comment,
            int level,
            int method,             
            const void* extrafield_local,
            uInt size_extrafield_local,
            const void* extrafield_global,
            uInt size_extrafield_global
            )
{
   zip_fileinfo zi;

   memset(&zi, 0, sizeof(zi));
   BOOL bOK = (buf_size != 0) && filetime(file,&zi.tmz_date,&zi.dosDate);
   if (bOK)
   {
      bOK = OpenNew(filenameinzip, &zi, comment, level, method, extrafield_local, 
         size_extrafield_local, extrafield_global, size_extrafield_global);
   }
   if (bOK)
   {
      for (size_t size_read;;)
      {
         size_read = fread(buf, 1, buf_size, file);
         if (size_read == 0)
         {
            break;
         }
         if (!WriteNew(buf,size_read))
         {
            break;
         }                    
      }
      bOK = CloseNew();
   }
   return bOK;
}

/////////////////////////////////////////////////////////////////////////////
